/*
 *  Definition of a simple read+CRC+write loop using crc32c
 *
 *  ICRAR - International Centre for Radio Astronomy Research
 *  (c) UWA - The University of Western Australia, 2014
 *  Copyright by UWA (in the framework of the ICRAR)
 *  All rights reserved
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA 02111-1307  USA
 *
 */

#ifndef _CRC32C_CONSUME_H_
#define _CRC32C_CONSUME_H_

#include "common.h"

#ifndef _IS_WINDOWS
int _crc32c_read_crc_write(int fd_in, int fd_out,
                           float in_timeout,
                           size_t buffsize, uint64_t total,
                           int crc_type, uint32_t* crc,
                           unsigned long *crc_time,
                           unsigned long *write_time);
#endif // _IS_WINDOWS

#endif
